import React from 'react'
import PropTypes from 'prop-types'
import Highlight from 'react-highlighter'
import memoize from 'memoize-one'
import TextInput from 'components/forms/TextInput'
import SelectWrapper from 'components/forms/private/SelectWrapper'
import { noop } from 'utils'

const optionFilter = (label, query) =>
  !query || label.toLowerCase().indexOf(query.toLowerCase()) >= 0

export default class Autocomplete extends React.Component {
  static propTypes = {
    allowNew: PropTypes.bool,
    allowNewDuplicate: PropTypes.bool,
    className: PropTypes.string,
    customInput: PropTypes.element,
    disabled: PropTypes.bool,
    forceNew: PropTypes.bool,
    invalid: PropTypes.bool,
    labelKey: PropTypes.string,
    name: PropTypes.string,
    newLabel: PropTypes.string,
    noClearOnEmpty: PropTypes.bool,
    noPortal: PropTypes.bool,
    onChange: PropTypes.func,
    onCreate: PropTypes.func,
    onKeyDown: PropTypes.func,
    onKeyUp: PropTypes.func,
    onKeyPress: PropTypes.func,
    onQueryChange: PropTypes.func,
    options: PropTypes.array,
    placeholder: PropTypes.string,
    renderOption: PropTypes.func,
    value: PropTypes.any,
    'data-test-id': PropTypes.string,
  }

  static defaultProps = {
    allowNew: true,
    labelKey: 'label',
    newLabel: "+ New '%value%'",
    onChange: noop,
    onCreate: noop,
    onQueryChange: noop,
    options: [],
  }

  state = {
    query: '',
    open: false,
    changedAfterOpen: false,
  }

  blur = () => this.input && this.input.blur()
  focus = () => this.input && this.input.focus()
  select = () => this.input && this.input.select()

  addNewOption = (options, parentOption = this.props) => {
    const { query } = this.state
    const { allowNew, allowNewDuplicate, forceNew, newLabel, group } = parentOption

    if (!allowNew) {
      return
    }

    if (!query && !forceNew) {
      return
    }

    const isDuplicate = options.some(o => o.label === query)
    if (isDuplicate && !allowNewDuplicate) {
      return
    }

    options.push({
      label: (newLabel || this.props.newLabel).replace('%value%', query),
      data: query,
      value: `create_new_${group || this.props.name}`,
      group,
      isNew: true,
    })
  }

  mapOption = option => ({
    ...option,
    label: option[this.props.labelKey],
    isGroup: Array.isArray(option.options),
  })

  getOptions = memoize((_options, query) => {
    const options = _options.map(this.mapOption).reduce((res, option, i) => {
      if (option.isGroup) {
        res.push(option)
        const children = option.options
          .map(this.mapOption)
          .filter(o => optionFilter(o.label, query))

        this.addNewOption(children, option)

        Array.prototype.push.apply(res, children)
      } else {
        if (optionFilter(option.label, query)) {
          res.push(option)
        }
      }

      return res
    }, [])

    this.addNewOption(options)

    return options
  })

  getValueLabel = () => {
    const { value, options } = this.props
    if (!value) {
      return ''
    }
    const option = options
      .reduce(
        (res, o) => (o.options ? Array.prototype.push.apply(res, o.options) : res.push(o), res),
        [],
      )
      .find(o => o.value === value)
    return option ? option.label : value
  }

  handleInputRef = el => (this.input = el)

  handleInputChange = e => {
    const { onQueryChange, name } = this.props
    const { value: query } = e.target
    if (!query && !this.props.noClearOnEmpty) {
      this.props.onChange({ target: { value: null, name } })
    }
    this.setState({ query, open: true, changedAfterOpen: true })
    onQueryChange(e)
  }

  handleInputBlur = e => this.setState({ open: false })

  handleInputFocus = e =>
    this.setState({ open: true, changedAfterOpen: false, query: this.getValueLabel() })

  handleRequestClose = (reason, e) => {
    if (reason === 'clickAnchor') {
      return
    }
    if (reason === 'esc') {
      this.setState({ open: false })
    }

    if (reason !== 'offScreen') {
      e.stopPropagationToLayers()
    }
  }

  handleSelect = option => {
    if (option.isNew) {
      this.props.onCreate({ target: { value: option.data, group: option.group, option, name } })
    } else {
      this.props.onChange({ target: { value: option.value, option, name: this.props.name } })
    }
    this.setState({ open: false, query: '' })
  }

  renderOption = option => {
    const { renderOption } = this.props
    const { query } = this.state

    const label = (
      <Highlight matchClass="font-semibold" search={query}>
        {option.label}
      </Highlight>
    )

    if (renderOption) {
      return renderOption(option, label)
    }

    return <div className="p-x p-y-sm">{label}</div>
  }

  render() {
    const { changedAfterOpen, query, open } = this.state
    const {
      className,
      customInput,
      disabled,
      invalid,
      name,
      onKeyDown,
      onKeyUp,
      onKeyPress,
      placeholder,
      value,
      ...rest
    } = this.props

    const options = this.getOptions(this.props.options, open && changedAfterOpen ? query : '')
    const inputValue = open ? query : this.getValueLabel()

    const inputProps = {
      ref: this.handleInputRef,
      value: inputValue,
      className,
      disabled,
      invalid,
      name,
      placeholder,
      autocomplete: false,
      onChange: this.handleInputChange,
      onFocus: this.handleInputFocus,
      onBlur: this.handleInputBlur,
      onKeyDown,
      onKeyUp,
      onKeyPress,
      'data-test-id': this.props['data-test-id'],
    }

    return (
      <SelectWrapper
        {...rest}
        open={open}
        anchor={this.input}
        options={options}
        onRequestClose={this.handleRequestClose}
        onSelect={this.handleSelect}
        className="m-y-xs"
        renderOption={this.renderOption}
        value={value}
      >
        {customInput ? React.cloneElement(customInput, inputProps) : <TextInput {...inputProps} />}
      </SelectWrapper>
    )
  }
}
