import React, { PureComponent } from 'react'
import PropTypes from 'prop-types'
import cx from 'classnames'
import { noop } from 'utils'
import { formatDate, parseDate, isValidDate } from 'utils/date'
import { KEYBOARD_ARROW_LEFT, KEYBOARD_ARROW_RIGHT } from 'constants/icons'
import SvgIcon from 'components/misc/SvgIcon'
import Calendar from 'components/forms/date/Calendar'
import DateInput from 'components/forms/date/DateInput'

const PRESETS = {
  today: 'Today',
  yesterday: 'Yesterday',
  week: 'Last 7 days',
  month: 'Last 30 days',
}

const INPUT_STYLE = { maxWidth: 120, height: 32 }

const startOfMonthString = v => formatDate(parseDate(v)).slice(0, -2) + '01'

export default class DateRange extends PureComponent {
  static propTypes = {
    calendars: PropTypes.number,
    controls: PropTypes.node,
    minDate: PropTypes.string,
    maxDate: PropTypes.string,
    name: PropTypes.string,
    value: PropTypes.arrayOf(PropTypes.string),
    onChange: PropTypes.func,
  }

  static defaultProps = {
    calendars: 2,
    onChange: noop,
  }

  static parseValue = value => {
    const [start, end] = Array.isArray(value) ? value.map(v => (isValidDate(v) ? v : '')) : ['', '']
    return { start, end }
  }

  state = { ...DateRange.parseValue(this.props.value), viewDate: new Date() }

  startInput = React.createRef()
  endInput = React.createRef()

  componentDidUpdate = (prevProps, prevState) => {
    const { value } = this.props
    if (prevProps.value !== this.props.value) {
      this.setState(DateRange.parseValue(value))
    }
  }

  setLimits(start, end, updateShowDate) {
    const { minDate, maxDate, name, onChange } = this.props
    if (start && end && start > end) {
      ;[start, end] = [end, start]
    }
    if (start && isValidDate(minDate) && start < minDate) {
      start = formatDate(parseDate(minDate))
    }
    if (end && isValidDate(maxDate) && end > maxDate) {
      end = formatDate(parseDate(maxDate))
    }
    const newState = { start, end }
    if (updateShowDate) {
      newState.viewDate = parseDate(end) || new Date()
    }
    this.setState(newState)
    if (start && end) {
      onChange({ target: { value: [start, end], name } })
    }
  }

  setViewMonthRelative = num => {
    const viewDate = new Date(this.state.viewDate)
    viewDate.setMonth(viewDate.getMonth() + num)
    this.setState({ viewDate })
  }

  handlePresetChange = e => {
    const preset = e.target.dataset.id
    let start, end
    switch (preset) {
      case 'today':
        start = formatDate(new Date())
        end = formatDate(new Date())
        break
      case 'yesterday':
        {
          const date = new Date()
          date.setDate(date.getDate() - 1)
          start = formatDate(date)
          end = formatDate(date)
        }
        break
      case 'week':
        {
          const date = new Date()
          date.setDate(date.getDate() - 6)
          start = formatDate(date)
          end = formatDate(new Date())
        }
        break
      case 'month':
        {
          const date = new Date()
          date.setDate(date.getDate() - 29)
          start = formatDate(date)
          end = formatDate(new Date())
        }
        break
    }
    this.setLimits(start, end, true)
  }

  handleMonthClick = e => {
    const date = parseDate(e.currentTarget.dataset.date)
    const start = new Date(date)
    start.setDate(1)
    const end = new Date(date)
    end.setMonth(end.getMonth() + 1)
    end.setDate(0)
    this.setLimits(formatDate(start), formatDate(end))
  }

  handleCalendarChange = e => {
    const { value } = e.target
    const { start: currentStart, end: currentEnd } = this.state
    const isStart = !currentStart || isValidDate(currentEnd)
    const start = isStart ? value : currentStart
    const end = isStart ? '' : value
    this.setLimits(start, end)

    this[`${isStart ? 'end' : 'start'}Input`].current.select()
  }

  handleChange = (limit, e) => {
    const { start, end } = this.state
    const limits = { start, end }
    limits[limit] = e.target.value
    this.setLimits(limits.start, limits.end, true)
  }

  handleNextMonthClick = () => this.setViewMonthRelative(1)
  handlePreviousMonthClick = () => this.setViewMonthRelative(-1)

  render() {
    const { start, end, viewDate } = this.state
    const { minDate, maxDate, controls } = this.props

    var tmpMonth = parseDate(viewDate)
    var calendars = []
    for (let i = 0; i < this.props.calendars; i++) {
      const dateString = startOfMonthString(tmpMonth)
      const disabled =
        (isValidDate(minDate) && dateString < startOfMonthString(minDate)) ||
        (isValidDate(maxDate) && dateString > maxDate)

      calendars.unshift(
        <div className="col p-t-sm p-x-xs" key={'m-' + i}>
          <div
            onClick={disabled ? undefined : this.handleMonthClick}
            data-date={dateString}
            className={cx(
              'p-y-sm text-center no-select',
              disabled ? 'text-disabled' : 'text-primary bg-hover pointer',
            )}
          >
            {tmpMonth.toLocaleDateString(undefined, { month: 'long', year: 'numeric' })}
          </div>
          <div className="p-x-xs p-b-xs">
            <Calendar
              onChange={this.handleCalendarChange}
              viewDate={dateString}
              hideHeader
              minDate={minDate}
              maxDate={maxDate}
              value={[start, end]}
            />
          </div>
        </div>,
      )
      tmpMonth.setMonth(tmpMonth.getMonth() - 1)
    }

    return (
      <div>
        <div className="row middle gutter p-a-sm">
          <div className="col-auto">
            {Object.keys(PRESETS).map(pName => {
              return (
                <a
                  data-id={pName}
                  key={pName}
                  className="p-x-xs p-y-sm bg-hover text-primary"
                  onClick={this.handlePresetChange}
                >
                  {PRESETS[pName]}
                </a>
              )
            })}
          </div>

          <div className="col">
            <div className="row middle">
              <div className="col-auto">
                <DateInput
                  ref={this.startInput}
                  value={this.state.start}
                  onChange={this.handleChange.bind(this, 'start')}
                  style={INPUT_STYLE}
                />
              </div>
              <div className="col-auto p-x-xs">
                <span>—</span>
              </div>
              <div className="col-auto">
                <DateInput
                  ref={this.endInput}
                  value={this.state.end}
                  onChange={this.handleChange.bind(this, 'end')}
                  style={INPUT_STYLE}
                />
              </div>
            </div>
          </div>

          <div className="col-auto">{controls}</div>
        </div>
        <div className="row b-t">
          <div
            className="col-auto p-x-xs p-y bg-hover text-secondary text-h-primary pointer"
            onClick={this.handlePreviousMonthClick}
          >
            <SvgIcon i={KEYBOARD_ARROW_LEFT} size={24} />
          </div>
          {calendars}
          <div
            className="col-auto p-x-xs p-y bg-hover text-secondary text-h-primary pointer"
            onClick={this.handleNextMonthClick}
          >
            <SvgIcon i={KEYBOARD_ARROW_RIGHT} size={24} />
          </div>
        </div>
      </div>
    )
  }
}
