import cm from './SelectWrapper.module.styl'
import React from 'react'
import PropTypes from 'prop-types'
import ReactHK from 'react-hotkeys'
import Scrollbars from 'react-custom-scrollbars'
import cx from 'classnames'
import memoize from 'memoize-one'
import Popover from 'components/layers/Popover'

export default class SelectWrapper extends React.PureComponent {
  static propTypes = {
    open: PropTypes.bool,
    anchor: PropTypes.any,
    children: PropTypes.any,
    emptyLabel: PropTypes.string,
    noPortal: PropTypes.bool,
    options: PropTypes.array,
    onSelect: PropTypes.func,
    onRequestClose: PropTypes.func,
    renderOption: PropTypes.func,
    value: PropTypes.any,
  }

  static defaultProps = {
    emptyLabel: 'No Items Found',
    renderOption: option => <div className="p-x p-y-sm">{option.label}</div>,
  }

  state = {
    index: -1,
  }

  getUngroupedOptions = memoize(options => options.filter(o => !o.isGroup))

  isOpen = () => this.props.open || this.props.noPortal

  componentDidMount = () => {
    if (this.isOpen()) {
      this.updateIndex()
    }
  }

  componentDidUpdate(prevProps, prevState, snapshot) {
    const { options, noPortal, open } = this.props
    if (options !== prevProps.options || (!noPortal && open !== prevProps.open)) {
      this.updateIndex()
    }
  }

  updateIndex = () => {
    const { options, value } = this.props
    const wrapper = this.optionsWrapper || {}
    wrapper.scrollTop = 0

    const withoutGroups = this.getUngroupedOptions(options)
    if (!this.isOpen() || !withoutGroups.length) {
      return this.setState({ index: -1 })
    }
    const index = withoutGroups.findIndex(o => o.value === value)
    this.setState({ index }, this.scrollOptionIntoView)
  }

  scrollOptionIntoView = () => {
    const { index } = this.state

    if (index >= 0 && this.optionsWrapper) {
      const option = this.optionsWrapper.querySelector(`[data-index="${index}"]`)

      const optionTop = option.offsetTop
      const optionBottom = optionTop + option.clientHeight
      const viewTop = this.optionsWrapper.scrollTop
      const viewBottom = viewTop + this.optionsWrapper.clientHeight

      if (optionBottom > viewBottom) {
        // Option is below view
        this.optionsWrapper.scrollTop = optionTop
      } else if (optionTop < viewTop) {
        // Option is above view
        this.optionsWrapper.scrollTop = optionBottom - this.optionsWrapper.clientHeight
      }
    }
  }

  handleKeydown = (e, key) => {
    e.preventDefault()
    const { index } = this.state
    const { options } = this.props
    const withoutGroups = this.getUngroupedOptions(options)

    if (withoutGroups.length) {
      if (key === 'up') {
        this.setState(
          { index: (index > 0 ? index : withoutGroups.length) - 1 },
          this.scrollOptionIntoView,
        )
      }
      if (key === 'down') {
        const newIndex = index + 1
        this.setState(
          { index: newIndex >= withoutGroups.length ? 0 : newIndex },
          this.scrollOptionIntoView,
        )
      }
    }

    if (key === 'enter') {
      const option = withoutGroups[index]
      if (option) {
        this.props.onSelect(option)
      }
    }
  }

  handleOptionClick = e => {
    const withoutGroups = this.getUngroupedOptions(this.props.options)
    this.props.onSelect(withoutGroups[+e.currentTarget.dataset.index])
  }

  handleOptionMouseEnter = e => this.setState({ index: +e.currentTarget.dataset.index })

  handleOptionMouseDown = e => e.preventDefault()

  handleOptionsWrapperRef = el => {
    if (el) {
      this.optionsWrapper = el.view || el
    }
  }

  handleOptionsMouseLeave = e => this.setState({ index: -1 })

  renderOptions() {
    const { index } = this.state
    const { emptyLabel, options, renderOption } = this.props

    let currentIndex = -1

    return options.map((option, i) => {
      if (option.isGroup) {
        const emptyGroup = !options[i + 1] || options[i + 1].isGroup
        return (
          <div key={i} className={`${cm.group} p-t-sm p-b-xs p-x text-secondary`}>
            <span className="text-sm">{option.label}</span>
            {emptyGroup && <div className="p-y-sm">{option.emptyLabel || emptyLabel}</div>}
          </div>
        )
      } else {
        currentIndex++
        return (
          <div
            key={i}
            onClick={this.handleOptionClick}
            onMouseDown={this.handleOptionMouseDown}
            onMouseEnter={this.handleOptionMouseEnter}
            className={cx('pointer', { 'bg-primary-light2': index === currentIndex })}
            data-index={currentIndex}
          >
            {renderOption(option)}
          </div>
        )
      }
    })
  }

  render() {
    const {
      anchor,
      children,
      emptyLabel,
      noPortal,
      open,
      onRequestClose,
      options,
      ...rest
    } = this.props
    const withoutGroups = this.getUngroupedOptions(options)

    const content = (
      <Scrollbars
        autoHeight={noPortal ? false : true}
        autoHeightMax={275}
        autoHide={false}
        ref={this.handleOptionsWrapperRef}
        className={noPortal ? 'flex-grow' : undefined}
      >
        {withoutGroups.length ? (
          <div className="p-y-xs" onMouseLeave={this.handleOptionsMouseLeave}>
            {this.renderOptions()}
          </div>
        ) : (
          <div className="p-y-sm p-x text-secondary">{emptyLabel}</div>
        )}
      </Scrollbars>
    )

    return (
      <ReactHK.HotKeys
        handlers={{
          up: this.handleKeydown,
          down: this.handleKeydown,
          esc: this.handleKeydown,
          enter: this.handleKeydown,
        }}
        className={noPortal ? cm.noPortal : undefined}
      >
        {children}
        {noPortal ? (
          content
        ) : (
          <Popover
            {...rest}
            anchor={anchor}
            at="bottom"
            autoPosition="shift flip"
            open={open}
            onRequestClose={onRequestClose}
            useLayerForClickAway={false}
            fitTargetWidth
            noContentPadding
          >
            {content}
          </Popover>
        )}
      </ReactHK.HotKeys>
    )
  }
}
