import cm from './PopoverBase.module.styl'
import React from 'react'
import ReactDOM from 'react-dom'
import PropTypes from 'prop-types'
import cx from 'classnames'
import { DOM, getWindow, noop } from 'utils'
import { parsePosition } from 'utils/geometry'
import RenderToLayer from 'components/layers/RenderToLayer'
import { getPopoverPosition } from './positioning'
import EventListener from 'utils/listener'
import { AXIS_STYLE_PROP, AutoPositionTypes, AutoPositionType } from '../constants'

const window = getWindow()

const getThresholds = element => {
  const width = window.innerWidth
  const height = window.innerHeight
  return { x: [0, width], y: [0, height] }
}

const getAnchorElement = anchor => {
  if (anchor == null) {
    return null
  }
  // Repeate of undocumented 'ReactDOM.findDOMNode' logic
  if (anchor.nodeType === DOM.HTMLNodeType.ELEMENT_NODE) {
    return anchor
  }

  return ReactDOM.findDOMNode(anchor) // eslint-disable-line
}

const parseAutoPosition = value => {
  if (value === false) {
    return { x: AutoPositionType.NONE, y: AutoPositionType.NONE }
  }
  if (!value || typeof value !== 'string') {
    return { x: AutoPositionType.AUTO, y: AutoPositionType.AUTO }
  }
  const values = value
    .trim()
    .split(' ')
    .map(v => v.trim())
  const x = AutoPositionTypes.find(t => t === values[0]) || AutoPositionType.AUTO
  const y = AutoPositionTypes.find(t => t === values[1]) || x
  return { x, y }
}

export default class PopoverBase extends React.PureComponent {
  static propTypes = {
    anchor: PropTypes.any,
    anchorOrigin: PropTypes.string,
    autoPosition: PropTypes.any,
    children: PropTypes.any,
    className: PropTypes.string,
    fitTargetWidth: PropTypes.bool,
    /** TODO */
    fullscreen: PropTypes.bool,
    onAxisAutoPositon: PropTypes.func,
    onRequestClose: PropTypes.func,
    open: PropTypes.bool,
    popoverOrigin: PropTypes.string,
    popoverRef: PropTypes.func,
    useLayerForClickAway: PropTypes.bool,
    style: PropTypes.object,
  }

  static defaultProps = {
    anchorOrigin: 'bottom center',
    onAxisAutoPositon: noop,
    onRequestClose: noop,
    open: false,
    popoverOrigin: 'top center',
    popoverRef: noop,
  }

  static getDerivedStateFromProps(nextProps, prevState) {
    const { anchor: propsAnchor, anchorOrigin: ao, autoPosition: ap, popoverOrigin: po } = nextProps
    const anchor = getAnchorElement(propsAnchor)

    return {
      anchor,
      anchorScrollableParents: [window, ...DOM.getScrollableParents(anchor)],
      anchorOrigin: parsePosition(ao),
      autoPosition: parseAutoPosition(ap),
      popoverOrigin: parsePosition(po),
    }
  }

  state = {}

  componentDidUpdate(prevProps, prevState, snapshot) {
    this.setPosition()
  }

  applyCoordinate = (axis, data) => {
    const cacheProp = `${axis}Cache`
    const styleProp = AXIS_STYLE_PROP[axis]
    const cache = this[cacheProp] || {}

    this[cacheProp] = data
    this.root.style[styleProp] = `${data.value}px`

    if (cache.position !== data.position) {
      this.props.onAxisAutoPositon(axis, data)
    }
  }

  setPosition = () => {
    const { root, thresholds } = this
    const { anchor, anchorOrigin, autoPosition, popoverOrigin } = this.state
    const { open, fitTargetWidth } = this.props

    // Positioning closed popover with offscreen anchor leads to constant updates
    // because of 'onRequestClose('offScreen')' calls
    // Root can be null when calling setPosition from cDU while opening popover
    if (!open || !anchor || !root) {
      return
    }

    const anchorRect = anchor.getBoundingClientRect()
    const popoverRect = root.getBoundingClientRect()

    if (this.isAnchorOffscreen(anchorRect, thresholds)) {
      return this.props.onRequestClose('offScreen')
    }

    const options = { autoPosition, thresholds }

    let { x, y } = getPopoverPosition(anchorOrigin, anchorRect, popoverOrigin, popoverRect, options)

    this.applyCoordinate('x', x)
    this.applyCoordinate('y', y)

    if (fitTargetWidth) {
      this.root.style.width = `${anchor.offsetWidth}px`
    }
  }

  isAnchorOffscreen(anchorRect, thresholds) {
    if (this.props.fullscreen) {
      return false
    }
    return (
      anchorRect.right < thresholds.x[0] ||
      anchorRect.left > thresholds.x[1] ||
      anchorRect.bottom < thresholds.y[0] ||
      anchorRect.top > thresholds.y[1]
    )
  }

  handleScroll = e => this.setPosition()
  handleResize = e => this.setPosition()
  handleTransitionEnd = e => this.setPosition()

  handleLayerRef = el => (this.layer = el)

  handleRootRef = el => {
    if (el) {
      this.root = el
      this.thresholds = getThresholds(el)
      this.setPosition()
      this.props.popoverRef(el)
    }
  }

  handleClickAway = (event, mousedownEvent) => {
    const { onRequestClose } = this.props
    if (event) {
      // if (this.props.useLayerForClickAway) {
      //   event.stopPropagationToLayers()
      //   event.preventDefault()
      // }

      const { anchor } = this.state
      if (
        anchor &&
        (DOM.isClickInElement(event, anchor) || DOM.isClickInElement(mousedownEvent, anchor))
      ) {
        return onRequestClose('clickAnchor', event)
      }
    }
    onRequestClose('clickAway', event)
  }

  handleEscapeKeyDown = e => this.props.onRequestClose('esc', e)

  render() {
    const { anchorScrollableParents } = this.state
    const {
      anchor, // eslint-disable-line no-unused-vars
      anchorOrigin, // eslint-disable-line no-unused-vars
      autoPosition, // eslint-disable-line no-unused-vars
      children,
      className,
      fitTargetWidth, // eslint-disable-line no-unused-vars
      fullscreen, // eslint-disable-line no-unused-vars
      onAxisAutoPositon, // eslint-disable-line no-unused-vars
      onRequestClose, // eslint-disable-line no-unused-vars
      open,
      popoverOrigin, // eslint-disable-line no-unused-vars
      popoverRef, // eslint-disable-line no-unused-vars
      style,
      ...rest
    } = this.props

    return (
      <span style={{ display: 'none' }}>
        {open &&
          anchorScrollableParents.map((el, index) => {
            return (
              <EventListener
                key={index}
                target={el}
                onScroll={this.handleScroll}
                onResize={this.handleResize}
              />
            )
          })}
        <EventListener target="document" onTransitionEnd={this.handleTransitionEnd} />
        <RenderToLayer
          {...rest}
          ref={this.handleLayerRef}
          open={open}
          onEscapeKeyDown={this.handleEscapeKeyDown}
          onClickAway={this.handleClickAway}
        >
          <div ref={this.handleRootRef} className={cx(cm.root, className)} style={style}>
            {children}
          </div>
        </RenderToLayer>
      </span>
    )
  }
}
